@@ -7,7 +7,7 @@ use Kavorka::Signature::Parameter ();
 package Kavorka::Signature;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.000_06';
+our $VERSION   = '0.000_07';
 
 use Parse::Keyword {};
 use Parse::KeywordX;
@@ -178,8 +178,9 @@ sub injections
 	or @slurpy && $slurpy[0]->name =~ /\A\%/
 	or @slurpy && $slurpy[0]->name =~ /\A\$/ && $slurpy[0]->type->is_a_type_of(Types::Standard::HashRef()))
 	{
+		require Data::Alias;
 		my @allowed_names = map +($_=>1), map @{$_->named_names}, @named;
-		$str .= sprintf('local %%_; { use warnings FATAL => qw(all); %%_ = @_[ %d .. $#_ ] };', 1 + $self->last_position).qq[];
+		$str .= sprintf('local %%_; { use warnings FATAL => qw(all); Data::Alias::alias(%%_ = @_[ %d .. $#_ ]) };', 1 + $self->last_position).qq[];
 		unless (@slurpy or $self->yadayada)
 		{
 			$str .= sprintf('{ my %%OK = (%s); ', join q[,], map sprintf('%s=>1,', B::perlstring $_), @allowed_names);
